@@ -217,7 +217,8 @@ goc_offscreen_box_realize (GtkWidget *widget)
 	window = gdk_window_new (gtk_widget_get_parent_window (widget),
 	                         &attributes, attributes_mask);
 	gtk_widget_set_window (widget, window);
-	gdk_window_set_composited (window, TRUE);
+	if (gdk_screen_is_composited (gdk_window_get_screen (window)))
+		gdk_window_set_composited (window, TRUE);
 	gdk_window_set_user_data (window, widget);
 
 	g_signal_connect (window, "pick-embedded-child",
@@ -233,7 +234,7 @@ goc_offscreen_box_realize (GtkWidget *widget)
 		attributes.width = child_area.width;
 		attributes.height = child_area.height;
 	}
-	offscreen_box->offscreen_window = gdk_window_new (gtk_widget_get_root_window (widget),
+	offscreen_box->offscreen_window = gdk_window_new (gdk_screen_get_root_window (gtk_widget_get_screen (widget)),
 	                                                  &attributes, attributes_mask);
 	gdk_window_set_user_data (offscreen_box->offscreen_window, widget);
 	if (offscreen_box->child)
@@ -260,6 +261,13 @@ goc_offscreen_box_unrealize (GtkWidget *widget)
 {
 	GocOffscreenBox *offscreen_box = GOC_OFFSCREEN_BOX (widget);
 
+	gdk_offscreen_window_set_embedder (offscreen_box->offscreen_window, NULL);
+
+	if (offscreen_box->child) {
+		gtk_widget_unrealize (offscreen_box->child);
+		gtk_widget_set_parent_window (offscreen_box->child, NULL);
+	}
+
 	gdk_window_set_user_data (offscreen_box->offscreen_window, NULL);
 	gdk_window_destroy (offscreen_box->offscreen_window);
 	offscreen_box->offscreen_window = NULL;
